/*
// Created by Lucile on 25/02/2021.
*/

/*!
 * \file personnage.c
 * \brief Character Source File
 * \authors Lucile
 * \version 1
 * \date 02/03/2021
 *
 * Character function implementation.
 *
 */

#include <stdio.h>
#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>
#include "personnage.h"
#include "prof.h"
#include "DS.h"
#include "MAP.h"
#define WINDOW_WIDTH 700
#define WINDOW_HEIGHT 700


/*!
 * \fn int init_pers(Personnage * character, int x, int y, int length, int width, int speed, char * path)
 * \brief This function initialize the character : position of the character's top left corner (x,y), size (length, width), speed and also the path to the character's image.
 *
 * \param [in] character a pointer on the character
 * \param [in] x the character's abscissa
 * \param [in] y the character's ordinate
 * \param [in] length the character's length
 * \param [in] width the character's width
 * \param [in] path the path to the character's image
 * \return 1 if the initialization went right.
 */

void init_pers(Personnage * character, int x, int y, int length, int width, int speed, char * path){
    character->x_pers=x;
    character->y_pers=y;
    character->long_pers=length;
    character->larg_pers=width;
    character->speed_pers=speed;
    character->image_pers = malloc(strlen(path));
    strcpy(character->image_pers, path);
}

void calculpointspers(Personnage * character){
	//calculate 3 others points of rectangle
	character->x_hr = character->x_pers + character->larg_pers ;
	character->y_hr = character->y_pers;
	character->x_ll = character->x_pers;
	character->y_ll = character->y_pers + character->long_pers;
	character->x_lr = character->x_pers + character->larg_pers;
	character->y_lr = character->y_pers + character->long_pers;
}

//Collisions Functions

int testposition(Personnage * pers, Prof * prof, DS * exam, MAP * map){
    //test variables
    int testbounds = testboundscol(pers); ;
    int testprof = testprofcol(pers,prof);

    //test bounds collision

    if(testbounds) return 4;

	//test prof collision

    if(testprof) return 3;

    //return the map test position

    return testmapcol(pers,map,exam); // = 0 if nothing , 1 if wall, 2 if exam

}

int testboundscol(Personnage * pers){
    if((pers->x_pers <= 0) || (pers->y_pers <= 0) || (pers->x_pers >= WINDOW_WIDTH - pers->larg_pers) || (pers->y_pers >= WINDOW_HEIGHT - pers->long_pers)) {
        return 1;
    }
    return 0;
}

int testprofcol(Personnage * pers, Prof * prof){

	int testhl;
	int testhr;
	int testll;
	int testlr;


	testhl=testpointwithprof(pers->x_pers, pers->y_pers,prof);
	testhr=testpointwithprof(pers->x_hr, pers->y_hr,prof) ;
	testll=testpointwithprof(pers->x_ll, pers->y_ll,prof);
	testlr=testpointwithprof(pers->x_lr, pers->y_lr,prof) ;

	if(testhl){
		return 1;
	}
	else if (testhr){
		return 1;
	}
	else if (testll){
		return 1;
	}
	else if (testlr){
		return 1;
	}
    return 0;
}


int testpointwithprof(int x, int y, Prof *prof){
	if (( x > prof->x_prof) && (x<prof->x_prof+prof->larg_prof)) {
        if (( y > prof->y_prof) && (y < prof->y_prof + prof->long_prof)) {
            return 1;
        }
    }
	return 0;
}



int testmapcol(Personnage * pers,  MAP * map, DS * exam ){
	int testhl = testpointwithmap(pers->x_pers,pers->y_pers,map,exam);
	int testhr = testpointwithmap(pers->x_hr,pers->y_hr,map,exam) ;
	int testll = testpointwithmap(pers->x_ll,pers->y_ll,map,exam) ;
	int testlr = testpointwithmap(pers->x_lr,pers->y_lr,map,exam) ;
	
	if(testhl==2 || testhr==2 || testll==2 || testlr==2) {
		return 2;
	}
	else if(testhl==1 || testhr==1 || testll==1 || testlr==1){
		return 1;
	}
	else return 0;
}


